@@ -1,4 +1,4 @@
-# Copyright (C) 2011-2013 2ndQuadrant Italia (Devise.IT S.r.L.)
+# Copyright (C) 2011-2014 2ndQuadrant Italia (Devise.IT S.r.L.)
 #
 # This file is part of Barman.
 #
@@ -15,146 +15,191 @@
 # You should have received a copy of the GNU General Public License
 # along with Barman.  If not, see <http://www.gnu.org/licenses/>.
 
-''' This module implements the interface with the command line
-and the logger.
-'''
+"""
+This module implements the interface with the command line and the logger.
+"""
 
 from argh import ArghParser, named, arg, expects_obj
-from barman.lockfile import lockfile
+from argparse import SUPPRESS
+from barman import output
+from barman.infofile import BackupInfo
+from barman import lockfile
 from barman.server import Server
 import barman.config
 import logging
 import os
 import sys
-from barman.backup import BackupInfo
+from barman.utils import drop_privileges, configure_logging, parse_log_level
 
 _logger = logging.getLogger(__name__)
 
-@named('list-server')
-@arg('--minimal', help='machine readable output', action='store_true')
-@expects_obj
-def list_server(args):
-    """ List available servers, with useful information
-    """
-    for name in sorted(barman.__config__.server_names()):
-        server = barman.__config__.get_server(name)
-        if server.description and not args.minimal:
-            yield "%s - %s" % (name, server.description)
-        else:
-            yield name
 
-@expects_obj
-def cron(args):
-    """ Run maintenance tasks
+@named('list-server')
+@arg('--minimal', help='machine readable output')
+def list_server(minimal=False):
+    """
+    List available servers, with useful information
+    """
+    servers = get_server_list()
+    for name in sorted(servers):
+        server = servers[name]
+        output.init('list_server', name, minimal=minimal)
+        output.result('list_server', name, server.config.description)
+    output.close_and_exit()
+
+
+def cron(verbose=True):
+    """
+    Run maintenance tasks
     """
     filename = os.path.join(barman.__config__.barman_home, '.cron.lock')
-    with lockfile(filename) as locked:
-        if not locked:
-            yield "ERROR: Another cron is running"
-            raise SystemExit, 1
-        else:
-            servers = [ Server(conf) for conf in barman.__config__.servers()]
+    try:
+        with lockfile.LockFile(filename, raise_if_fail=True):
+            servers = [Server(conf) for conf in barman.__config__.servers()]
             for server in servers:
-                for lines in server.cron(verbose=True):
+                for lines in server.cron(verbose):
                     yield lines
+    except lockfile.LockFileBusy:
+        raise SystemExit("ERROR: Another cron is running")
 
+    except lockfile.LockFilePermissionDenied:
+        raise SystemExit("ERROR: Permission denied, unable to access '%s'"
+                         % filename)
+
+
+# noinspection PyUnusedLocal
 def server_completer(prefix, parsed_args, **kwargs):
     global_config(parsed_args)
     for conf in barman.__config__.servers():
-         if conf.name.startswith(prefix):
-             yield conf.name
+        if conf.name.startswith(prefix) \
+                and conf.name not in parsed_args.server_name:
+            yield conf.name
 
+
+# noinspection PyUnusedLocal
 def server_completer_all(prefix, parsed_args, **kwargs):
-    print >> sys.stderr,  "INVOKED"
     global_config(parsed_args)
     for conf in barman.__config__.servers():
-        if conf.name.startswith(prefix):
+        if conf.name.startswith(prefix) \
+                and conf.name not in parsed_args.server_name:
             yield conf.name
-    if 'all'.startswith(prefix):
+    if 'server_name' in parsed_args \
+            and parsed_args.server_name is None \
+            and 'all'.startswith(prefix):
         yield 'all'
 
+
+# noinspection PyUnusedLocal
 def backup_completer(prefix, parsed_args, **kwargs):
     global_config(parsed_args)
     server = get_server(parsed_args)
-    if server:
-        for backup_id in sorted(server.get_available_backups().iterkeys(), reverse=True):
+    if server and len(parsed_args.backup_id) == 0:
+        for backup_id in sorted(server.get_available_backups().iterkeys(),
+                                reverse=True):
             if backup_id.startswith(prefix):
-                 yield backup_id
+                yield backup_id
+        for special_id in ('latest', 'last', 'oldest', 'first'):
+            if len(server.get_available_backups()) > 0 \
+                    and special_id.startswidth(prefix):
+                yield special_id
     else:
         return
 
+
 @arg('server_name', nargs='+',
      completer=server_completer_all,
-     help="specifies the server names for the backup command ('all' will show all available servers)")
+     help="specifies the server names for the backup command "
+          "('all' will show all available servers)")
+@arg('--immediate-checkpoint',
+     help='forces the initial checkpoint to be done as quickly as possible',
+     dest='immediate_checkpoint',
+     action='store_true',
+     default=SUPPRESS)
+@arg('--no-immediate-checkpoint',
+     help='forces the initial checkpoint to be spreaded',
+     dest='immediate_checkpoint',
+     action='store_false',
+     default=SUPPRESS)
 @expects_obj
 def backup(args):
-    """ Perform a full backup for the given server
+    """
+    Perform a full backup for the given server
     """
     servers = get_server_list(args)
-    ok = True
     for name in sorted(servers):
         server = servers[name]
-        if server == None:
-            yield "Unknown server '%s'" % (name)
-            ok = False
+        if server is None:
+            output.error("Unknown server '%s'" % name)
             continue
-        try:
-            for line in server.backup():
-                yield line
-        except:
-            ok = False
-        yield ''
-    if not ok:
-        raise SystemExit(1)
+        immediate_checkpoint = getattr(args, 'immediate_checkpoint',
+                                       server.config.immediate_checkpoint)
+        server.backup(immediate_checkpoint)
+    output.close_and_exit()
+
 
 @named('list-backup')
 @arg('server_name', nargs='+',
      completer=server_completer_all,
-     help="specifies the server name for the command ('all' will show all available servers)")
+     help="specifies the server name for the command "
+          "('all' will show all available servers)")
 @arg('--minimal', help='machine readable output', action='store_true')
 @expects_obj
 def list_backup(args):
-    """ List available backups for the given server (supports 'all')
+    """
+    List available backups for the given server (supports 'all')
     """
     servers = get_server_list(args)
-    ok = True
     for name in sorted(servers):
         server = servers[name]
-        if server == None:
-            yield "Unknown server '%s'" % (name)
-            ok = False
+        output.init('list_backup', name, minimal=args.minimal)
+        if server is None:
+            output.error("Unknown server '%s'" % name)
             continue
-        if not args.minimal:
-            for line in server.list_backups():
-                yield line
-        else:
-            for backup_id in sorted(server.get_available_backups().iterkeys(), reverse=True):
-                yield backup_id
-    if not ok:
-        raise SystemExit(1)
+        server.list_backups()
+    output.close_and_exit()
+
 
 @arg('server_name', nargs='+',
      completer=server_completer_all,
      help='specifies the server name for the command')
 @expects_obj
 def status(args):
-    """ Shows live information and status of the PostgreSQL server
+    """
+    Shows live information and status of the PostgreSQL server
+    """
+    servers = get_server_list(args)
+    for name in sorted(servers):
+        server = servers[name]
+        if server is None:
+            output.error("Unknown server '%s'" % name)
+            continue
+        output.init('status', name)
+        server.status()
+    output.close_and_exit()
+
+
+@arg('server_name', nargs='+',
+     completer=server_completer_all,
+     help='specifies the server name for the command')
+@expects_obj
+def rebuild_xlogdb(args):
+    """
+    Rebuild the WAL file database guessing it from the disk content.
     """
     servers = get_server_list(args)
     ok = True
     for name in sorted(servers):
         server = servers[name]
-        if server == None:
+        if server is None:
             yield "Unknown server '%s'" % (name)
             ok = False
             continue
-        for line in server.status():
+        for line in server.rebuild_xlogdb():
             yield line
         yield ''
     if not ok:
         raise SystemExit(1)
 
-
 @arg('server_name',
      completer=server_completer,
      help='specifies the server name for the command')
@@ -162,7 +207,8 @@ def status(args):
 @arg('--target-time',
      help='target time. You can use any valid unambiguous representation. e.g: "YYYY-MM-DD HH:MM:SS.mmm"')
 @arg('--target-xid', help='target transaction ID')
-@arg('--target-name', help='target name created previously with pg_create_restore_point() function call')
+@arg('--target-name',
+     help='target name created previously with pg_create_restore_point() function call')
 @arg('--exclusive',
      help='set target xid to be non inclusive', action="store_true")
 @arg('--tablespace',
@@ -171,8 +217,8 @@ def status(args):
 @arg('--remote-ssh-command',
      metavar='SSH_COMMAND',
      help='This options activates remote recovery, by specifying the secure shell command '
-     'to be launched on a remote host. It is the equivalent of the "ssh_command" server'
-     'option in the configuration file for remote recovery. Example: "ssh postgres@db2"')
+          'to be launched on a remote host. It is the equivalent of the "ssh_command" server'
+          'option in the configuration file for remote recovery. Example: "ssh postgres@db2"')
 @arg('backup_id',
      help='specifies the backup ID to recover')
 @arg('destination_directory',
@@ -182,36 +228,36 @@ def recover(args):
     """ Recover a server at a given time or xid
     """
     server = get_server(args)
-    if server == None:
+    if server is None:
         raise SystemExit("ERROR: unknown server '%s'" % (args.server_name))
-    # Retrieves the backup info
-    backup_id = parse_backup_id(server, args)
-    backup = server.get_backup(backup_id)
-    if backup == None or backup.status != BackupInfo.DONE:
-        raise SystemExit("ERROR: unknown backup '%s' for server '%s'" % (args.backup_id, args.server_name))
-    # decode the tablespace relocation rules
+    # Retrieves the backup
+    backup = parse_backup_id(server, args)
+    if backup is None or backup.status != BackupInfo.DONE:
+        raise SystemExit("ERROR: unknown backup '%s' for server '%s'" % (
+            args.backup_id, args.server_name))
+        # decode the tablespace relocation rules
     tablespaces = {}
     if args.tablespace:
         for rule in args.tablespace:
             try:
                 tablespaces.update([rule.split(':', 1)])
             except:
-                raise SystemExit("ERROR: invalid tablespace relocation rule '%s'\n"
-                                 "HINT: the valid syntax for a relocation rule is NAME:LOCATION" % rule)
-    # validate the rules against the tablespace list
-    valid_tablespaces = [tablespace_data[0] for tablespace_data in backup.tablespaces] if backup.tablespaces else []
+                raise SystemExit(
+                    "ERROR: invalid tablespace relocation rule '%s'\n"
+                    "HINT: the valid syntax for a relocation rule is NAME:LOCATION" % rule)
+                # validate the rules against the tablespace list
+    valid_tablespaces = [tablespace_data[0] for tablespace_data in
+                         backup.tablespaces] if backup.tablespaces else []
     for tablespace in tablespaces:
         if tablespace not in valid_tablespaces:
             raise SystemExit("ERROR: invalid tablespace name '%s'\n"
-                             "HINT: please use any of the following tablespaces: %s" 
+                             "HINT: please use any of the following tablespaces: %s"
                              % (tablespace, ', '.join(valid_tablespaces)))
-    # explicitly disallow the rsync remote syntax (common mistake)
+            # explicitly disallow the rsync remote syntax (common mistake)
     if ':' in args.destination_directory:
         raise SystemExit(
             "ERROR: the destination directory parameter cannot contain the ':' character\n"
             "HINT: if you want to do a remote recovery you have to use the --remote-ssh-command option")
-    if args.remote_ssh_command and len(tablespaces) > 0:
-        raise SystemExit("ERROR: Tablespace relocation is not supported with remote recovery")
     for line in server.recover(backup,
                                args.destination_directory,
                                tablespaces=tablespaces,
@@ -221,68 +267,55 @@ def recover(args):
                                target_name=args.target_name,
                                exclusive=args.exclusive,
                                remote_command=args.remote_ssh_command
-                               ):
+    ):
         yield line
 
+
 @named('show-server')
 @arg('server_name', nargs='+',
      completer=server_completer_all,
      help="specifies the server names to show ('all' will show all available servers)")
 @expects_obj
 def show_server(args):
-    """ Show all configuration parameters for the specified servers
+    """
+    Show all configuration parameters for the specified servers
     """
     servers = get_server_list(args)
-    ok = True
     for name in sorted(servers):
         server = servers[name]
-        if server == None:
-            yield "Unknown server '%s'" % (name)
-            ok = False
+        if server is None:
+            output.error("Unknown server '%s'" % name)
             continue
-        for line in server.show():
-            yield line
-        yield ''
-    if not ok:
-        raise SystemExit(1)
+        output.init('show_server', name)
+        server.show()
+    output.close_and_exit()
+
 
 @arg('server_name', nargs='+',
      completer=server_completer_all,
-     help="specifies the server names to check ('all' will check all available servers)")
+     help="specifies the server names to check "
+          "('all' will check all available servers)")
 @arg('--nagios', help='Nagios plugin compatible output', action='store_true')
 @expects_obj
 def check(args):
-    """ Check if the server configuration is working.
-    This function returns 0 if every checks pass, or 0 if any of these fails
     """
-    servers = get_server_list(args)
-    issues = 0
-    for name in sorted(servers):
-        ok = True
-        server = servers[name]
-        if server == None:
-            if not args.nagios:
-                yield "Unknown server '%s'" % (name)
-            issues += 1
-            continue
-        for line, status in server.check():
-            ok &= status
-            if not args.nagios:
-                yield line
-        if not args.nagios:
-            yield ''
-        if not ok:
-            issues += 1
+    Check if the server configuration is working.
 
+    This command returns success if every checks pass,
+    or failure if any of these fails
+    """
     if args.nagios:
-        if issues:
-            yield "BARMAN CRITICAL - %d server out of %d that have been checked has issues" % (issues, len(servers))
-            raise SystemExit(2)
-        else:
-            yield "BARMAN OK - Ready to serve the Espresso backup"
+        output.set_output_writer(output.NagiosOutputWriter())
+    servers = get_server_list(args)
+    for name in sorted(servers):
+        server = servers[name]
+        if server is None:
+            output.error("Unknown server '%s'" % name)
+            continue
+        output.init('check', name)
+        server.check()
+    output.close_and_exit()
 
-    if issues:
-        raise SystemExit(1)
 
 @named('show-backup')
 @arg('server_name',
@@ -293,20 +326,22 @@ def check(args):
      help='specifies the backup ID')
 @expects_obj
 def show_backup(args):
-    """ This method Shows a single backup information
+    """
+    This method shows a single backup information
     """
     server = get_server(args)
-    if server == None:
-        yield "Unknown server '%s'" % (args.server_name)
-        return
-    # Retrieves the backup info
-    backup_id = parse_backup_id(server, args)
-    backup = server.get_backup(backup_id)
-    if backup == None:
-        yield "Unknown backup '%s' for server '%s'" % (args.backup_id, args.server_name)
-        return
-    for line in backup.show():
-        yield line
+    if server is None:
+        output.error("Unknown server '%s'" % args.server_name)
+    else:
+        # Retrieves the backup
+        backup_info = parse_backup_id(server, args)
+        if backup_info is None:
+            output.error("Unknown backup '%s' for server '%s'" % (
+                args.backup_id, args.server_name))
+        else:
+            server.show_backup(backup_info)
+    output.close_and_exit()
+
 
 @named('list-files')
 @arg('server_name',
@@ -315,30 +350,32 @@ def show_backup(args):
 @arg('backup_id',
      completer=backup_completer,
      help='specifies the backup ID')
-@arg('--target', choices=('standalone', 'data', 'wal', 'full'), default='standalone',
+@arg('--target', choices=('standalone', 'data', 'wal', 'full'),
+     default='standalone',
      help='''
      Possible values are: data (just the data files), standalone (base backup files, including required WAL files),
      wal (just WAL files between the beginning of base backup and the following one (if any) or the end of the log) and
      full (same as data + wal). Defaults to %(default)s
      '''
-     )
+)
 @expects_obj
 def list_files(args):
     """ List all the files for a single backup
     """
     server = get_server(args)
-    if server == None:
+    if server is None:
         yield "Unknown server '%s'" % (args.server_name)
         return
-    # Retrieves the backup info
-    backup_id = parse_backup_id(server, args)
-    backup = server.get_backup(backup_id)
-    if backup == None:
-        yield "Unknown backup '%s' for server '%s'" % (args.backup_id, args.server_name)
+    # Retrieves the backup
+    backup = parse_backup_id(server, args)
+    if backup is None:
+        yield "Unknown backup '%s' for server '%s'" % (
+            args.backup_id, args.server_name)
         return
     for line in backup.get_list_of_files(args.target):
         yield line
 
+
 @arg('server_name',
      completer=server_completer,
      help='specifies the server name for the command')
@@ -350,14 +387,14 @@ def delete(args):
     """ Delete a backup
     """
     server = get_server(args)
-    if server == None:
+    if server is None:
         yield "Unknown server '%s'" % (args.server_name)
         return
-    # Retrieves the backup info
-    backup_id = parse_backup_id(server, args)
-    backup = server.get_backup(backup_id)
-    if backup == None:
-        yield "Unknown backup '%s' for server '%s'" % (args.backup_id, args.server_name)
+    # Retrieves the backup
+    backup = parse_backup_id(server, args)
+    if backup is None:
+        yield "Unknown backup '%s' for server '%s'" % (
+            args.backup_id, args.server_name)
         return
     for line in server.delete_backup(backup):
         yield line
@@ -366,6 +403,7 @@ def delete(args):
 class stream_wrapper(object):
     """ This class represents a wrapper for a stream
     """
+
     def __init__(self, stream):
         self.stream = stream
 
@@ -376,10 +414,12 @@ class stream_wrapper(object):
     def __getattr__(self, attr):
         return getattr(self.stream, attr)
 
+
 _output_stream = stream_wrapper(sys.stdout)
 
+
 def global_config(args):
-    ''' Set the configuration file '''
+    """ Set the configuration file """
     if hasattr(args, 'config'):
         filename = args.config
     else:
@@ -387,26 +427,67 @@ def global_config(args):
             filename = os.environ['BARMAN_CONFIG_FILE']
         except KeyError:
             filename = None
-    barman.__config__ = barman.config.Config(filename)
+    config = barman.config.Config(filename)
+    barman.__config__ = config
+
+    # change user if needed
+    try:
+        drop_privileges(config.user)
+    except OSError:
+        msg = "ERROR: please run barman as %r user" % config.user
+        raise SystemExit(msg)
+    except KeyError:
+        msg = "ERROR: the configured user %r does not exists" % config.user
+        raise SystemExit(msg)
+
+    # configure logging
+    log_level = parse_log_level(config.log_level)
+    configure_logging(config.log_file,
+                      log_level or barman.config.DEFAULT_LOG_LEVEL,
+                      config.log_format)
+    if log_level is None:
+        _logger.warn('unknown log_level in config file: %s', config.log_level)
+
+    # configure output
+    if args.format != output.DEFAULT_WRITER:
+        output.set_output_writer(args.format)
+
+    # Load additional configuration files
+    _logger.debug('Loading additional configuration files')
+    config.load_configuration_files_directory()
+
     _logger.debug('Initialized Barman version %s (config: %s)',
-                 barman.__version__, barman.__config__.config_file)
+                  barman.__version__, config.config_file)
     if hasattr(args, 'quiet') and args.quiet:
-        _logger.debug("Replacing output stream ")
+        _logger.debug("Replacing output stream")
         global _output_stream
         _output_stream.set_stream(open(os.devnull, 'w'))
 
 
 def get_server(args):
-    ''' Get the server from the configuration '''
+    """
+    Get a single server from the configuration
+
+    :param args: an argparse namespace containing a single server_name parameter
+    """
     config = barman.__config__.get_server(args.server_name)
     if not config:
         return None
     return Server(config)
 
-def get_server_list(args):
-    ''' Get the server list from the configuration '''
-    if args.server_name[0] == 'all':
-        return dict((conf.name, Server(conf)) for conf in barman.__config__.servers())
+
+def get_server_list(args=None):
+    """
+    Get the server list from the configuration
+
+    If args the parameter is None or arg.server_name[0] is 'all'
+    returns all defined servers
+
+    :param args: an argparse namespace containing a list server_name parameter
+    """
+    if args is None or args.server_name[0] == 'all':
+        return dict(
+            (conf.name, Server(conf)) for conf in barman.__config__.servers())
     else:
         server_dict = {}
         for server in args.server_name:
@@ -417,25 +498,41 @@ def get_server_list(args):
                 server_dict[server] = Server(conf)
         return server_dict
 
+
 def parse_backup_id(server, args):
-    ''' Parses special backup IDs such as latest, oldest, etc. '''
+    """
+    Parses backup IDs including special words such as latest, oldest, etc.
+
+    :param Server server: server object to search for the required backup
+    :param args: command lien arguments namespace
+    :rtype BackupInfo,None: the decoded backup_info object
+    """
     if args.backup_id in ('latest', 'last'):
         backup_id = server.get_last_backup()
     elif args.backup_id in ('oldest', 'first'):
         backup_id = server.get_first_backup()
     else:
-        return args.backup_id
-    if backup_id == None:
-        raise SystemExit("ERROR: '%s' backup is not available for server '%s'" % (args.backup_id, args.server_name))
-    return backup_id
+        backup_id = args.backup_id
+    backup_info = server.get_backup(backup_id) if backup_id else None
+    return backup_info
 
 
 def main():
-    ''' The main method of Barman '''
+    """
+    The main method of Barman
+    """
     p = ArghParser()
-    p.add_argument('-v', '--version', action='version', version=barman.__version__)
-    p.add_argument('-c', '--config', help='uses a configuration file (defaults: $HOME/.barman.conf, /etc/barman.conf)')
+    p.add_argument('-v', '--version', action='version',
+                   version=barman.__version__)
+    p.add_argument('-c', '--config',
+                   help='uses a configuration file '
+                        '(defaults: %s)'
+                        % ', '.join(barman.config.Config.CONFIG_FILES),
+                   default=SUPPRESS)
     p.add_argument('-q', '--quiet', help='be quiet', action='store_true')
+    p.add_argument('-f', '--format', help='output format',
+                   choices=output.AVAILABLE_WRITERS.keys(),
+                   default=output.DEFAULT_WRITER)
     p.add_commands(
         [
             cron,
@@ -449,14 +546,26 @@ def main():
             list_files,
             recover,
             delete,
+            rebuild_xlogdb,
         ]
     )
+    # noinspection PyBroadException
     try:
         p.dispatch(pre_call=global_config, output_file=_output_stream)
     except Exception:
-        msg = "ERROR: Unhandled exception. See log file for more details."
-        logging.exception(msg)
-        raise SystemExit(msg)
+        msg = "Unhandled exception. See log file for more details."
+        output.exception(msg)
+        output.close_and_exit()
+
 
 if __name__ == '__main__':
+    # This code requires the mock module and allow us to test
+    # bash completion inside the IDE debugger
+    try:
+        import mock
+        sys.stdout = mock.Mock(wraps=sys.stdout)
+        sys.stdout.isatty.return_value = True
+        os.dup2(2, 8)
+    except ImportError:
+        pass
     main()
