@@ -1,4 +1,4 @@
-# Copyright (C) 2011-2013 2ndQuadrant Italia (Devise.IT S.r.L.)
+# Copyright (C) 2011-2014 2ndQuadrant Italia (Devise.IT S.r.L.)
 #
 # This file is part of Barman.
 #
@@ -19,9 +19,10 @@
 Barman is able to manage multiple servers.
 '''
 
-from barman import xlog, _pretty_size
-from barman.lockfile import lockfile
-from barman.backup import BackupInfo, BackupManager
+from barman import xlog, output
+from barman.infofile import BackupInfo, UnknownBackupIdException
+from barman.lockfile import LockFile
+from barman.backup import BackupManager
 from barman.command_wrappers import Command
 from barman.retention_policies import RetentionPolicyFactory, SimpleWALRetentionPolicy
 import os
@@ -114,14 +115,14 @@ class Server(object):
                     'retention_policy', self.config.retention_policy)
                 # Reassign the configuration value (we keep it in one place)
                 self.config.retention_policy = rp
-                _logger.info('Retention policy for server %s: %s' % (
+                _logger.debug('Retention policy for server %s: %s' % (
                     self.config.name, self.config.retention_policy))
                 try:
                     rp = RetentionPolicyFactory.create(self,
                         'wal_retention_policy', self.config.wal_retention_policy)
                     # Reassign the configuration value (we keep it in one place)
                     self.wal_retention_policy = rp
-                    _logger.info('WAL retention policy for server %s: %s' % (
+                    _logger.debug('WAL retention policy for server %s: %s' % (
                         self.config.name, self.config.wal_retention_policy))
                 except:
                     _logger.error('Invalid wal_retention_policy setting "%s" for server "%s" (fallback to "main")' % (
@@ -135,115 +136,158 @@ class Server(object):
                 _logger.error('Invalid retention_policy setting "%s" for server "%s"' % (
                     self.config.retention_policy, self.config.name))
 
-    def check_retention_policy_settings(self):
-        '''Checks retention policy settings'''
-        if self.config.retention_policy and not self.enforce_retention_policies:
-            yield ("\tretention policy settings: FAILED (see log)", False)
-        else:
-            yield ("\tretention policy settings: OK", True)
-
-    def check_ssh(self):
-        '''Checks SSH connection'''
-        cmd = Command(self.ssh_command, self.ssh_options)
-        ret = cmd("true")
-        if ret == 0:
-            yield ("\tssh: OK", True)
-        else:
-            yield ("\tssh: FAILED (return code: %s)" % (ret,), False)
-
-    def check_postgres(self):
-        '''
-        Checks PostgreSQL connection
-        '''
-        remote_status = self.get_remote_status()
-        if remote_status['server_txt_version']:
-            yield ("\tPostgreSQL: OK", True)
-        else:
-            yield ("\tPostgreSQL: FAILED", False)
-            return
-        if remote_status['archive_mode'] == 'on':
-            yield ("\tarchive_mode: OK", True)
-        else:
-            yield ("\tarchive_mode: FAILED (please set it to 'on')", False)
-        if remote_status['archive_command'] and remote_status['archive_command'] != '(disabled)':
-            yield ("\tarchive_command: OK", True)
-        else:
-            yield ("\tarchive_command: FAILED (please set it accordingly to documentation)", False)
-
-    def check_directories(self):
-        '''Checks backup directories and creates them if they do not exist'''
-        error = None
-        try:
-            for key in self.config.KEYS:
-                if key.endswith('_directory') and hasattr(self.config, key):
-                    val = getattr(self.config, key)
-                    if val != None and not os.path.isdir(val):
-                        os.makedirs(val)
-        except OSError, e:
-                error = e.strerror
-        if not error:
-            yield ("\tdirectories: OK", True)
-        else:
-            yield ("\tdirectories: FAILED (%s)" % (error,), False)
-
     def check(self):
-        '''
+        """
         Implements the 'server check' command and makes sure SSH and PostgreSQL
         connections work properly. It checks also that backup directories exist
         (and if not, it creates them).
-        '''
-        status = [(("Server %s:" % (self.config.name,), True),)]
-        #FIXME: Description makes sense in a "check" command?
-        #if self.config.description: status.append(("\tdescription: %s" % (self.config.description),))
-        status.append(self.check_ssh())
-        status.append(self.check_postgres())
-        status.append(self.check_directories())
+        """
+        self.check_ssh()
+        self.check_postgres()
+        self.check_directories()
         # Check retention policies
-        status.append(self.check_retention_policy_settings())
+        self.check_retention_policy_settings()
         # Executes the backup manager set of checks
-        status.append(self.backup_manager.check())
+        self.backup_manager.check()
 
-        return itertools.chain.from_iterable(status)
+    def check_ssh(self):
+        """
+        Checks SSH connection
+        """
+        cmd = Command(self.ssh_command, self.ssh_options)
+        ret = cmd("true")
+        if ret == 0:
+            output.result('check', self.config.name, 'ssh', True)
+        else:
+            output.result('check', self.config.name, 'ssh', False,
+                          'return code: %s' % ret)
 
-    def status_postgres(self):
-        '''Status of PostgreSQL server'''
+    def check_postgres(self):
+        """
+        Checks PostgreSQL connection
+        """
         remote_status = self.get_remote_status()
         if remote_status['server_txt_version']:
-            yield "\tPostgreSQL version: %s " % (remote_status['server_txt_version'])
+            output.result('check', self.config.name, 'PostgreSQL', True)
         else:
-            yield "\tPostgreSQL version: FAILED trying to get PostgreSQL version"
+            output.result('check', self.config.name, 'PostgreSQL', False)
+            return
+        if remote_status['archive_mode'] == 'on':
+            output.result('check', self.config.name, 'archive_mode', True)
+        else:
+            output.result('check', self.config.name, 'archive_mode', False,
+                          "please set it to 'on'")
+        if remote_status['archive_command'] and\
+                remote_status['archive_command'] != '(disabled)':
+            output.result('check', self.config.name, 'archive_command', True)
+        else:
+            output.result('check', self.config.name, 'archive_command', False,
+                          'please set it accordingly to documentation')
+
+    def _make_directories(self):
+        """
+        Make backup directories in case they do not exist
+        """
+        for key in self.config.KEYS:
+            if key.endswith('_directory') and hasattr(self.config, key):
+                val = getattr(self.config, key)
+                if val is not None and not os.path.isdir(val):
+                    #noinspection PyTypeChecker
+                    os.makedirs(val)
+
+    def check_directories(self):
+        """
+        Checks backup directories and creates them if they do not exist
+        """
+        error = None
+        try:
+            self._make_directories()
+        except OSError, e:
+            output.result('check', self.config.name, 'directories', False,
+                          "%s: %s" % (e.filename, e.strerror))
+        else:
+            output.result('check', self.config.name, 'directories', True)
+
+    def check_retention_policy_settings(self):
+        """
+        Checks retention policy setting
+        """
+        if self.config.retention_policy and not self.enforce_retention_policies:
+            output.result('check', self.config.name,
+                          'retention policy settings', False, 'see log')
+        else:
+            output.result('check', self.config.name,
+                          'retention policy settings', True)
+
+    def status_postgres(self):
+        """
+        Status of PostgreSQL server
+        """
+        remote_status = self.get_remote_status()
+        if remote_status['server_txt_version']:
+            output.result('status', self.config.name,
+                          "pg_version",
+                          "PostgreSQL version",
+                          remote_status['server_txt_version'])
+        else:
+            output.result('status', self.config.name,
+                          "pg_version",
+                          "PostgreSQL version",
+                          "FAILED trying to get PostgreSQL version")
             return
         if remote_status['data_directory']:
-            yield "\tPostgreSQL Data directory: %s " % (remote_status['data_directory'])
-        if remote_status['archive_command']:
-            yield "\tarchive_command: %s " % (remote_status['archive_command'])
-        else:
-            yield "\tarchive_command: FAILED (please set it accordingly to documentation)"
-        if remote_status['last_shipped_wal']:
-            yield "\tarchive_status: last shipped WAL segment %s" % remote_status['last_shipped_wal']
-        else:
-            yield "\tarchive_status: No WAL segment shipped yet"
+            output.result('status', self.config.name,
+                          "data_directory",
+                          "PostgreSQL Data directory",
+                          remote_status['data_directory'])
+        output.result('status', self.config.name,
+                      "archive_command",
+                      "PostgreSQL 'archive_command' setting",
+                      remote_status['archive_command']
+                      or "FAILED (please set it accordingly to documentation)")
+        output.result('status', self.config.name,
+                      "last_shipped_wal",
+                      "Archive status",
+                      "last shipped WAL segment %s" %
+                      remote_status['last_shipped_wal']
+                      or "No WAL segment shipped yet")
         if remote_status['current_xlog']:
-            yield "\tcurrent_xlog: %s " % (remote_status['current_xlog'])
+            output.result('status', self.config.name,
+                          "current_xlog",
+                          "Current WAL segment",
+                          remote_status['current_xlog'])
 
     def status_retention_policies(self):
-        '''Status of retention policies enforcement'''
+        """
+        Status of retention policies enforcement
+        """
         if self.enforce_retention_policies:
-            yield "\tRetention policies: enforced (mode: %s, retention: %s, WAL retention: %s)" % (
-                self.config.retention_policy_mode, self.config.retention_policy,
-                self.config.wal_retention_policy)
+            output.result('status', self.config.name,
+                          "retention_policies",
+                          "Retention policies",
+                          "enforced "
+                          "(mode: %s, retention: %s, WAL retention: %s)" % (
+                              self.config.retention_policy_mode,
+                              self.config.retention_policy,
+                              self.config.wal_retention_policy))
         else:
-            yield "\tRetention policies: not enforced"
+            output.result('status', self.config.name,
+                          "retention_policies",
+                          "Retention policies",
+                          "not enforced")
 
     def status(self):
-        '''Implements the 'server status' command.'''
-        status = [("Server %s:" % (self.config.name),)]
-        if self.config.description: status.append(("\tdescription: %s" % (self.config.description),))
-        status.append(self.status_postgres())
-        status.append(self.status_retention_policies())
+        """
+        Implements the 'server-status' command.
+        """
+        if self.config.description:
+            output.result('status', self.config.name,
+                          "description",
+                          "Description", self.config.description)
+        self.status_postgres()
+        self.status_retention_policies()
         # Executes the backup manager status info method
-        status.append(self.backup_manager.status())
-        return itertools.chain.from_iterable(status)
+        self.backup_manager.status()
 
     def get_remote_status(self):
         '''Get the status of the remote server'''
@@ -266,6 +310,7 @@ class Server(object):
                     result['current_xlog'] = cur.fetchone()[0];
                 except:
                     result['current_xlog'] = None
+                result.update(self.get_pg_configuration_files())
         except:
             pass
         cmd = Command(self.ssh_command, self.ssh_options)
@@ -281,22 +326,17 @@ class Server(object):
         return result
 
     def show(self):
-        '''Shows the server configuration'''
-        yield "Server %s:" % (self.config.name)
-        for key in self.config.KEYS:
-            if hasattr(self.config, key):
-                yield "\t%s: %s" % (key, getattr(self.config, key))
+        """
+        Shows the server configuration
+        """
+        #Populate result map with all the required keys
+        result = dict([
+            (key, getattr(self.config, key))
+            for key in self.config.KEYS
+        ])
         remote_status = self.get_remote_status()
-        for key in remote_status:
-            yield "\t%s: %s" % (key, remote_status[key])
-
-        try:
-            cf = self.get_pg_configuration_files()
-            if cf:
-                for key in sorted(cf.keys()):
-                    yield "\t%s: %s" % (key, cf[key])
-        except:
-            yield "ERROR: cannot connect to the PostgreSQL Server"
+        result.update(remote_status)
+        output.result('show_server', self.config.name, result)
 
     @contextmanager
     def pg_connect(self):
@@ -358,12 +398,18 @@ class Server(object):
             except:
                 return None
 
-    def pg_start_backup(self):
-        '''Execute a pg_start_backup'''
+    def pg_start_backup(self, backup_label, immediate_checkpoint):
+        """
+        Execute a pg_start_backup
+
+        :param backup_label: label for the backup
+        :param immediate_checkpoint Boolean for immediate checkpoint execution
+        """
         with self.pg_connect() as conn:
             try:
                 cur = conn.cursor()
-                cur.execute('SELECT xlog_loc, (pg_xlogfile_name_offset(xlog_loc)).* from pg_start_backup(%s) as xlog_loc', ('Barman backup',))
+                cur.execute('SELECT xlog_loc, (pg_xlogfile_name_offset(xlog_loc)).* from pg_start_backup(%s,%s) as xlog_loc',
+                            (backup_label, immediate_checkpoint))
                 return cur.fetchone()
             except:
                 return None
@@ -385,9 +431,16 @@ class Server(object):
         '''
         return self.backup_manager.delete_backup(backup)
 
-    def backup(self):
+    def backup(self, immediate_checkpoint):
         '''Performs a backup for the server'''
-        return self.backup_manager.backup()
+        try:
+            # check required backup directories exist
+            self._make_directories()
+        except OSError, e:
+            output.error('failed to create %s directory: %s',
+                         e.filename, e.strerror)
+        else:
+            self.backup_manager.backup(immediate_checkpoint)
 
     def get_available_backups(self, status_filter=BackupManager.DEFAULT_STATUS_FILTER):
         '''Get a list of available backups
@@ -413,32 +466,26 @@ class Server(object):
         return self.backup_manager.get_first_backup(status_filter)
 
     def list_backups(self):
-        '''Lists all the available backups for the server'''
+        """
+        Lists all the available backups for the server
+        """
         status_filter = BackupInfo.STATUS_NOT_EMPTY
         retention_status = self.report_backups()
         backups = self.get_available_backups(status_filter)
         for key in sorted(backups.iterkeys(), reverse=True):
             backup = backups[key]
+
+            backup_size = 0
+            wal_size = 0
+            rstatus = None
             if backup.status == BackupInfo.DONE:
-                _, backup_wal_size, _, wal_until_next_size, _ = self.get_wal_info(backup)
-                backup_size = _pretty_size(backup.size or 0 + backup_wal_size)
-                wal_size = _pretty_size(wal_until_next_size)
-                end_time = backup.end_time.ctime()
-                rstatus = ''
-                if self.enforce_retention_policies and retention_status[backup.backup_id] != BackupInfo.VALID:
-                    rstatus = ' - %s' % retention_status[backup.backup_id]
-                if backup.tablespaces:
-                    tablespaces = [("%s:%s" % (name, location))for name, _, location in backup.tablespaces]
-                    yield ("%s %s - %s - Size: %s - WAL Size: %s (tablespaces: %s)%s"
-                           % (self.config.name, backup.backup_id,
-                              end_time, backup_size, wal_size,
-                              ', '.join(tablespaces), rstatus))
-                else:
-                    yield ("%s %s - %s - Size: %s - WAL Size: %s%s"
-                           % (self.config.name, backup.backup_id,
-                              end_time, backup_size, wal_size, rstatus))
-            else:
-                yield "%s %s - %s" % (self.config.name, backup.backup_id, backup.status)
+                wal_info = self.get_wal_info(backup)
+                backup_size = backup.size or 0 + wal_info['wal_size']
+                wal_size = wal_info['wal_until_next_size']
+                if self.enforce_retention_policies and \
+                        retention_status[backup.backup_id] != BackupInfo.VALID:
+                    rstatus = retention_status[backup.backup_id]
+            output.result('list_backup', backup, backup_size, wal_size, rstatus)
 
     def get_backup(self, backup_id):
         '''Return the backup information for the given backup,
@@ -516,29 +563,29 @@ class Server(object):
                 # count
                 yield (name, size)
 
-    def get_wal_info(self, backup):
-        '''Returns information about WALs for the given backup
+    def get_wal_info(self, backup_info):
+        """
+        Returns information about WALs for the given backup
 
-        :param backup: a backup object of which return wal information
-        '''
-        end = backup.end_wal
+        :param BackupInfo backup_info: the target backup
+        """
+        end = backup_info.end_wal
 
         # counters
-        wal_num = 0
-        wal_size = 0
-        wal_until_next_num = 0
-        wal_until_next_size = 0
-        wal_last = None
+        wal_info = dict.fromkeys(
+            ('wal_num', 'wal_size',
+             'wal_until_next_num', 'wal_until_next_size'), 0)
+        wal_info['wal_last'] = None
 
-        for name, size in self.get_wal_until_next_backup(backup):
+        for name, size in self.get_wal_until_next_backup(backup_info):
                 if name <= end:
-                    wal_num += 1
-                    wal_size += size
+                    wal_info['wal_num'] += 1
+                    wal_info['wal_size'] += size
                 else:
-                    wal_until_next_num += 1
-                    wal_until_next_size += size
-                wal_last = name
-        return wal_num, wal_size, wal_until_next_num, wal_until_next_size, wal_last
+                    wal_info['wal_until_next_num'] += 1
+                    wal_info['wal_until_next_size'] += size
+                wal_info['wal_last'] = name
+        return wal_info
 
     def recover(self, backup, dest, tablespaces=[], target_tli=None, target_time=None, target_xid=None, target_name=None, exclusive=False, remote_command=None):
         '''Performs a recovery of a backup'''
@@ -553,6 +600,20 @@ class Server(object):
 
     @contextmanager
     def xlogdb(self, mode='r'):
+        """
+        Context manager to access the xlogdb file.
+
+        This method uses locking to make sure only one process is accessing
+        the database at a time. The database file will be created if not exists.
+
+        Usage example:
+
+            with server.xlogdb('w') ad file:
+                file.write(new_line)
+
+        :param str mode: open the file with the required mode
+            (default read-only)
+        """
         if not os.path.exists(self.config.wals_directory):
             os.makedirs(self.config.wals_directory)
         xlogdb = os.path.join(self.config.wals_directory, self.XLOG_DB)
@@ -564,10 +625,18 @@ class Server(object):
             else:
                 mode = "a%s" % mode[1:]
         xlogdb_lock = xlogdb + ".lock"
-        with lockfile(xlogdb_lock, wait=True):
+        with LockFile(xlogdb_lock, wait=True):
             with open(xlogdb, mode) as f:
+
+                # execute the block nested in the with statement
                 yield f
 
+                # we are exiting the context
+                # make sure the data is written to disk
+                # http://docs.python.org/2/library/os.html#os.fsync
+                f.flush()
+                os.fsync(f.fileno())
+
     def xlogdb_parse_line(self, line):
         '''Parse a line from xlog catalogue
 
@@ -589,3 +658,61 @@ class Server(object):
             return dict()
         else:
             return self.config.retention_policy.report()
+
+    def rebuild_xlogdb(self):
+        """
+        Rebuild the whole xlog database guessing it from the archive content.
+        """
+        return self.backup_manager.rebuild_xlogdb()
+
+    def get_backup_ext_info(self, backup_info):
+        """
+        Return a dictionary containing all available information about a backup
+
+        The result is equivalent to the sum of information from
+
+         * BackupInfo object
+         * the Server.get_wal_info() return value
+         * the context in the catalog (if available)
+         * the retention policy status
+
+        :param backup_info: the target backup
+        :rtype dict: all information about a backup
+        """
+        backup_ext_info = backup_info.to_dict()
+        if backup_info.status == BackupInfo.DONE:
+            try:
+                previous_backup = self.backup_manager.get_previous_backup(
+                    backup_ext_info['backup_id'])
+                next_backup = self.backup_manager.get_next_backup(
+                    backup_ext_info['backup_id'])
+                if previous_backup:
+                    backup_ext_info[
+                        'previous_backup_id'] = previous_backup.backup_id
+                else:
+                    backup_ext_info['previous_backup_id'] = None
+                if next_backup:
+                    backup_ext_info['next_backup_id'] = next_backup.backup_id
+                else:
+                    backup_ext_info['next_backup_id'] = None
+            except UnknownBackupIdException:
+                # no next_backup_id and previous_backup_id items
+                # means "Not available"
+                pass
+            backup_ext_info.update(self.get_wal_info(backup_info))
+            if self.enforce_retention_policies:
+                policy = self.config.retention_policy
+                backup_ext_info['retention_policy_status'] = \
+                    policy.backup_status(backup_info.backup_id)
+            else:
+                backup_ext_info['retention_policy_status'] = None
+        return backup_ext_info
+
+    def show_backup(self, backup_info):
+        """
+        Output all available information about a backup
+
+        :param backup_info: the target backup
+        """
+        backup_ext_info = self.get_backup_ext_info(backup_info)
+        output.result('show_backup', backup_ext_info)
